﻿/*************************************\
 *          Netzwerkzugriffe         *
 *                                   *
 * Kapitel: Erweiterte               *
 *           Programmierthemen       *
 * Zuordnung: C#-Buch V2.0 14.1      *
 *                                   *
 *  Copyright 2015 by Benjamin Jung  *
\*************************************/
using System;
using System.Net;
using System.Net.Sockets;

namespace CSV20.Netzwerkzugriffe
{
    class Program
    {
        private static string sTitle = "SNTP-Client";
        private static string sSntpServer = "ptbtime1.ptb.de";

        static void Main(string[] args)
        {
            IPAddress oIpAddress;
            IPEndPoint oIpEndPoint;
            UdpClient oSocket = null;
            DateTime oDateTime; 
            byte[] aPacket = new byte[48];
            uint uiTimestamp;

            Console.Title = sTitle;
            Console.WriteLine(sTitle);
            Console.WriteLine();

            try
            {
                // Umwandlung von sSntpServer in eine IP-Adressen ausprobieren
                if (!IPAddress.TryParse(sSntpServer, out oIpAddress))
                {
                    // sSntpServer ist ein DNS-Name, starte Auflösung
                    oIpAddress = Dns.GetHostAddresses(sSntpServer)[0];
                    Console.WriteLine("DNS-Name {0} in {1} aufgelöst!", sSntpServer, oIpAddress.ToString());
                }

                // IP-Endpunkt erstellen
                oIpEndPoint = new IPEndPoint(oIpAddress, 123);

                // Socket öffnen
                oSocket = new UdpClient();
                oSocket.Connect(oIpEndPoint);

                // leeres Packet mit Kennezeichnung an Server senden
                aPacket[0] = 0x1B;              // NTP-Version 3, Client-Modus (siehe RFC 5905)
                oSocket.Send(aPacket, 48);

                // Daten vom Server abholen
                aPacket = oSocket.Receive(ref oIpEndPoint);
                
                // UTC-Sekunden aus NTP-Packet laden
                uiTimestamp = BitConverter.ToUInt32(aPacket, 40);
                uiTimestamp = (((uiTimestamp & 0x000000ff) << 24) +
                               ((uiTimestamp & 0x0000ff00) <<  8) +
                               ((uiTimestamp & 0x00ff0000) >>  8) +
                               ((uiTimestamp & 0xff000000) >> 24));

                // in Datum- / Uhrzeit-Objekt umwandeln und ausgeben
                oDateTime = new DateTime(1900, 1, 1).AddSeconds(uiTimestamp);
                Console.WriteLine("UTC-Zeit: " + oDateTime.ToString());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
            finally
            {
                // Socket schließen
                if (oSocket != null)
                    oSocket.Close();
            }

            Console.ReadKey();
        }
    }
}